from collections.abc import Mapping
from typing import Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="OllamaMessage")


@_attrs_define
class OllamaMessage:
    """
    Attributes:
        role (str):
        content (str):
        images (Union[None, Unset, list[str]]):
    """

    role: str
    content: str
    images: Union[None, Unset, list[str]] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        role = self.role

        content = self.content

        images: Union[None, Unset, list[str]]
        if isinstance(self.images, Unset):
            images = UNSET
        elif isinstance(self.images, list):
            images = self.images

        else:
            images = self.images

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "role": role,
                "content": content,
            }
        )
        if images is not UNSET:
            field_dict["images"] = images

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        role = d.pop("role")

        content = d.pop("content")

        def _parse_images(data: object) -> Union[None, Unset, list[str]]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                images_type_0 = cast(list[str], data)

                return images_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, list[str]], data)

        images = _parse_images(d.pop("images", UNSET))

        ollama_message = cls(
            role=role,
            content=content,
            images=images,
        )

        ollama_message.additional_properties = d
        return ollama_message

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.relation_request import RelationRequest
from ...models.relation_response import RelationResponse
from ...types import UNSET, Response, Unset


def _get_kwargs(
    source: str,
    target: str,
    *,
    body: RelationRequest,
    relation_type: str,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["relation_type"] = relation_type

    json_api_key_header_value: Union[None, Unset, str]
    if isinstance(api_key_header_value, Unset):
        json_api_key_header_value = UNSET
    else:
        json_api_key_header_value = api_key_header_value
    params["api_key_header_value"] = json_api_key_header_value

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "put",
        "url": f"/relations/{source}/{target}",
        "params": params,
    }

    _body = body.to_dict()

    _kwargs["json"] = _body
    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, RelationResponse]]:
    if response.status_code == 200:
        response_200 = RelationResponse.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, RelationResponse]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    source: str,
    target: str,
    *,
    client: AuthenticatedClient,
    body: RelationRequest,
    relation_type: str,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Response[Union[HTTPValidationError, RelationResponse]]:
    r"""Edit Relation

     Updates relation (edge) information in the knowledge graph and re-embeds the relation in the vector
    database.

    Args:
        source_entity: Name of the source entity
        target_entity: Name of the target entity
        updated_data: Dictionary containing updated attributes, e.g. {\"description\": \"new
    description\", \"keywords\": \"new keywords\"}

    Returns:
        Dictionary containing updated relation information

    Args:
        source (str):
        target (str):
        relation_type (str):
        api_key_header_value (Union[None, Unset, str]):
        body (RelationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, RelationResponse]]
    """

    kwargs = _get_kwargs(
        source=source,
        target=target,
        body=body,
        relation_type=relation_type,
        api_key_header_value=api_key_header_value,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    source: str,
    target: str,
    *,
    client: AuthenticatedClient,
    body: RelationRequest,
    relation_type: str,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Optional[Union[HTTPValidationError, RelationResponse]]:
    r"""Edit Relation

     Updates relation (edge) information in the knowledge graph and re-embeds the relation in the vector
    database.

    Args:
        source_entity: Name of the source entity
        target_entity: Name of the target entity
        updated_data: Dictionary containing updated attributes, e.g. {\"description\": \"new
    description\", \"keywords\": \"new keywords\"}

    Returns:
        Dictionary containing updated relation information

    Args:
        source (str):
        target (str):
        relation_type (str):
        api_key_header_value (Union[None, Unset, str]):
        body (RelationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, RelationResponse]
    """

    return sync_detailed(
        source=source,
        target=target,
        client=client,
        body=body,
        relation_type=relation_type,
        api_key_header_value=api_key_header_value,
    ).parsed


async def asyncio_detailed(
    source: str,
    target: str,
    *,
    client: AuthenticatedClient,
    body: RelationRequest,
    relation_type: str,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Response[Union[HTTPValidationError, RelationResponse]]:
    r"""Edit Relation

     Updates relation (edge) information in the knowledge graph and re-embeds the relation in the vector
    database.

    Args:
        source_entity: Name of the source entity
        target_entity: Name of the target entity
        updated_data: Dictionary containing updated attributes, e.g. {\"description\": \"new
    description\", \"keywords\": \"new keywords\"}

    Returns:
        Dictionary containing updated relation information

    Args:
        source (str):
        target (str):
        relation_type (str):
        api_key_header_value (Union[None, Unset, str]):
        body (RelationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, RelationResponse]]
    """

    kwargs = _get_kwargs(
        source=source,
        target=target,
        body=body,
        relation_type=relation_type,
        api_key_header_value=api_key_header_value,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    source: str,
    target: str,
    *,
    client: AuthenticatedClient,
    body: RelationRequest,
    relation_type: str,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Optional[Union[HTTPValidationError, RelationResponse]]:
    r"""Edit Relation

     Updates relation (edge) information in the knowledge graph and re-embeds the relation in the vector
    database.

    Args:
        source_entity: Name of the source entity
        target_entity: Name of the target entity
        updated_data: Dictionary containing updated attributes, e.g. {\"description\": \"new
    description\", \"keywords\": \"new keywords\"}

    Returns:
        Dictionary containing updated relation information

    Args:
        source (str):
        target (str):
        relation_type (str):
        api_key_header_value (Union[None, Unset, str]):
        body (RelationRequest):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, RelationResponse]
    """

    return (
        await asyncio_detailed(
            source=source,
            target=target,
            client=client,
            body=body,
            relation_type=relation_type,
            api_key_header_value=api_key_header_value,
        )
    ).parsed



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.merge_entities_request_merge_strategy_type_0 import (
        MergeEntitiesRequestMergeStrategyType0,
    )


T = TypeVar("T", bound="MergeEntitiesRequest")


@_attrs_define
class MergeEntitiesRequest:
    """
    Attributes:
        source_entities (list[str]): List of source entities to merge
        target_entity (str): Name of the target entity after merging
        merge_strategy (Union['MergeEntitiesRequestMergeStrategyType0', None, Unset]): Merge strategy for properties
            ('max', 'min', 'concat', 'first', 'last'). Example: {"description": "concat", "weight": "max"}
    """

    source_entities: list[str]
    target_entity: str
    merge_strategy: Union["MergeEntitiesRequestMergeStrategyType0", None, Unset] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.merge_entities_request_merge_strategy_type_0 import (
            MergeEntitiesRequestMergeStrategyType0,
        )

        source_entities = self.source_entities

        target_entity = self.target_entity

        merge_strategy: Union[None, Unset, dict[str, Any]]
        if isinstance(self.merge_strategy, Unset):
            merge_strategy = UNSET
        elif isinstance(self.merge_strategy, MergeEntitiesRequestMergeStrategyType0):
            merge_strategy = self.merge_strategy.to_dict()
        else:
            merge_strategy = self.merge_strategy

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "source_entities": source_entities,
                "target_entity": target_entity,
            }
        )
        if merge_strategy is not UNSET:
            field_dict["merge_strategy"] = merge_strategy

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.merge_entities_request_merge_strategy_type_0 import (
            MergeEntitiesRequestMergeStrategyType0,
        )

        d = dict(src_dict)
        source_entities = cast(list[str], d.pop("source_entities"))

        target_entity = d.pop("target_entity")

        def _parse_merge_strategy(
            data: object,
        ) -> Union["MergeEntitiesRequestMergeStrategyType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                merge_strategy_type_0 = MergeEntitiesRequestMergeStrategyType0.from_dict(data)

                return merge_strategy_type_0
            except:  # noqa: E722
                pass
            return cast(Union["MergeEntitiesRequestMergeStrategyType0", None, Unset], data)

        merge_strategy = _parse_merge_strategy(d.pop("merge_strategy", UNSET))

        merge_entities_request = cls(
            source_entities=source_entities,
            target_entity=target_entity,
            merge_strategy=merge_strategy,
        )

        merge_entities_request.additional_properties = d
        return merge_entities_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    label: str,
    max_depth: Union[Unset, int] = 3,
    min_degree: Union[Unset, int] = 0,
    inclusive: Union[Unset, bool] = False,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> dict[str, Any]:
    params: dict[str, Any] = {}

    params["label"] = label

    params["max_depth"] = max_depth

    params["min_degree"] = min_degree

    params["inclusive"] = inclusive

    json_api_key_header_value: Union[None, Unset, str]
    if isinstance(api_key_header_value, Unset):
        json_api_key_header_value = UNSET
    else:
        json_api_key_header_value = api_key_header_value
    params["api_key_header_value"] = json_api_key_header_value

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/graphs",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[Any, HTTPValidationError]]:
    if response.status_code == 200:
        response_200 = response.json()
        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[Any, HTTPValidationError]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    label: str,
    max_depth: Union[Unset, int] = 3,
    min_degree: Union[Unset, int] = 0,
    inclusive: Union[Unset, bool] = False,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Response[Union[Any, HTTPValidationError]]:
    """Get Knowledge Graph

     Retrieve a connected subgraph of nodes where the label includes the specified label.
    Maximum number of nodes is constrained by the environment variable `MAX_GRAPH_NODES` (default:
    1000).
    When reducing the number of nodes, the prioritization criteria are as follows:
        1. min_degree does not affect nodes directly connected to the matching nodes
        2. Label matching nodes take precedence
        3. Followed by nodes directly connected to the matching nodes
        4. Finally, the degree of the nodes
    Maximum number of nodes is limited to env MAX_GRAPH_NODES(default: 1000)

    Args:
        label (str): Label to get knowledge graph for
        max_depth (int, optional): Maximum depth of graph. Defaults to 3.
        inclusive_search (bool, optional): If True, search for nodes that include the label. Defaults to
    False.
        min_degree (int, optional): Minimum degree of nodes. Defaults to 0.

    Returns:
        Dict[str, List[str]]: Knowledge graph for label

    Args:
        label (str):
        max_depth (Union[Unset, int]):  Default: 3.
        min_degree (Union[Unset, int]):  Default: 0.
        inclusive (Union[Unset, bool]):  Default: False.
        api_key_header_value (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        label=label,
        max_depth=max_depth,
        min_degree=min_degree,
        inclusive=inclusive,
        api_key_header_value=api_key_header_value,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    label: str,
    max_depth: Union[Unset, int] = 3,
    min_degree: Union[Unset, int] = 0,
    inclusive: Union[Unset, bool] = False,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Get Knowledge Graph

     Retrieve a connected subgraph of nodes where the label includes the specified label.
    Maximum number of nodes is constrained by the environment variable `MAX_GRAPH_NODES` (default:
    1000).
    When reducing the number of nodes, the prioritization criteria are as follows:
        1. min_degree does not affect nodes directly connected to the matching nodes
        2. Label matching nodes take precedence
        3. Followed by nodes directly connected to the matching nodes
        4. Finally, the degree of the nodes
    Maximum number of nodes is limited to env MAX_GRAPH_NODES(default: 1000)

    Args:
        label (str): Label to get knowledge graph for
        max_depth (int, optional): Maximum depth of graph. Defaults to 3.
        inclusive_search (bool, optional): If True, search for nodes that include the label. Defaults to
    False.
        min_degree (int, optional): Minimum degree of nodes. Defaults to 0.

    Returns:
        Dict[str, List[str]]: Knowledge graph for label

    Args:
        label (str):
        max_depth (Union[Unset, int]):  Default: 3.
        min_degree (Union[Unset, int]):  Default: 0.
        inclusive (Union[Unset, bool]):  Default: False.
        api_key_header_value (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return sync_detailed(
        client=client,
        label=label,
        max_depth=max_depth,
        min_degree=min_degree,
        inclusive=inclusive,
        api_key_header_value=api_key_header_value,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    label: str,
    max_depth: Union[Unset, int] = 3,
    min_degree: Union[Unset, int] = 0,
    inclusive: Union[Unset, bool] = False,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Response[Union[Any, HTTPValidationError]]:
    """Get Knowledge Graph

     Retrieve a connected subgraph of nodes where the label includes the specified label.
    Maximum number of nodes is constrained by the environment variable `MAX_GRAPH_NODES` (default:
    1000).
    When reducing the number of nodes, the prioritization criteria are as follows:
        1. min_degree does not affect nodes directly connected to the matching nodes
        2. Label matching nodes take precedence
        3. Followed by nodes directly connected to the matching nodes
        4. Finally, the degree of the nodes
    Maximum number of nodes is limited to env MAX_GRAPH_NODES(default: 1000)

    Args:
        label (str): Label to get knowledge graph for
        max_depth (int, optional): Maximum depth of graph. Defaults to 3.
        inclusive_search (bool, optional): If True, search for nodes that include the label. Defaults to
    False.
        min_degree (int, optional): Minimum degree of nodes. Defaults to 0.

    Returns:
        Dict[str, List[str]]: Knowledge graph for label

    Args:
        label (str):
        max_depth (Union[Unset, int]):  Default: 3.
        min_degree (Union[Unset, int]):  Default: 0.
        inclusive (Union[Unset, bool]):  Default: False.
        api_key_header_value (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[Any, HTTPValidationError]]
    """

    kwargs = _get_kwargs(
        label=label,
        max_depth=max_depth,
        min_degree=min_degree,
        inclusive=inclusive,
        api_key_header_value=api_key_header_value,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    label: str,
    max_depth: Union[Unset, int] = 3,
    min_degree: Union[Unset, int] = 0,
    inclusive: Union[Unset, bool] = False,
    api_key_header_value: Union[None, Unset, str] = UNSET,
) -> Optional[Union[Any, HTTPValidationError]]:
    """Get Knowledge Graph

     Retrieve a connected subgraph of nodes where the label includes the specified label.
    Maximum number of nodes is constrained by the environment variable `MAX_GRAPH_NODES` (default:
    1000).
    When reducing the number of nodes, the prioritization criteria are as follows:
        1. min_degree does not affect nodes directly connected to the matching nodes
        2. Label matching nodes take precedence
        3. Followed by nodes directly connected to the matching nodes
        4. Finally, the degree of the nodes
    Maximum number of nodes is limited to env MAX_GRAPH_NODES(default: 1000)

    Args:
        label (str): Label to get knowledge graph for
        max_depth (int, optional): Maximum depth of graph. Defaults to 3.
        inclusive_search (bool, optional): If True, search for nodes that include the label. Defaults to
    False.
        min_degree (int, optional): Minimum degree of nodes. Defaults to 0.

    Returns:
        Dict[str, List[str]]: Knowledge graph for label

    Args:
        label (str):
        max_depth (Union[Unset, int]):  Default: 3.
        min_degree (Union[Unset, int]):  Default: 0.
        inclusive (Union[Unset, bool]):  Default: False.
        api_key_header_value (Union[None, Unset, str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[Any, HTTPValidationError]
    """

    return (
        await asyncio_detailed(
            client=client,
            label=label,
            max_depth=max_depth,
            min_degree=min_degree,
            inclusive=inclusive,
            api_key_header_value=api_key_header_value,
        )
    ).parsed



"""
Configuration module for RAG Agent MCP server.

Supports two authentication modes:
1. Service Key mode: Only --api-key is provided (admin privileges, no JWT needed)
2. Anon Key + JWT mode: --api-key (anon key) + --user + --user-password provided
   - Calls /login endpoint to obtain JWT token
   - User-level permissions

Authentication type is automatically determined:
- Only --api-key provided → Service Key mode (admin)
- --api-key + --user + --user-password provided → Anon Key + JWT mode (user-level)
"""

import argparse
import re
from urllib.parse import urlparse

DEFAULT_HOST = "localhost"
DEFAULT_PORT = 9621
DEFAULT_API_KEY = ""


def parse_args():
    """Parse command line arguments for RAG Agent MCP server."""
    parser = argparse.ArgumentParser(description="RAG Agent MCP Server")
    parser.add_argument(
        "--host", 
        default=DEFAULT_HOST, 
        help=f"Supabase URL or RAG Agent API host (default: {DEFAULT_HOST})"
    )
    parser.add_argument(
        "--port",
        type=int,
        default=DEFAULT_PORT,
        help=f"API port (default: {DEFAULT_PORT})",
    )
    parser.add_argument(
        "--api-key", 
        default=DEFAULT_API_KEY, 
        help="Supabase anon key or RAG Agent API key (required)"
    )
    parser.add_argument(
        "--user",
        default=None,
        help="Supabase Auth user (email or username for admin)"
    )
    parser.add_argument(
        "--user-password",
        default=None,
        help="Supabase Auth user password"
    )
    return parser.parse_args()


args = parse_args()

# Basic configuration
LIGHTRAG_API_HOST = args.host
LIGHTRAG_API_PORT = args.port
LIGHTRAG_API_KEY = args.api_key

# User authentication configuration
AUTH_USER = args.user
AUTH_USER_PASSWORD = args.user_password

# Determine authentication mode
# - "service_key": Service Key mode (admin privileges, only api-key needed)
# - "jwt": Anon Key + JWT mode (user-level, requires login)
if AUTH_USER and AUTH_USER_PASSWORD and LIGHTRAG_API_KEY:
    # User credentials provided → JWT mode
    AUTH_MODE = "jwt"
else:
    # Only api-key provided → Service Key mode
    AUTH_MODE = "service_key"

# Legacy compatibility (for backward compatibility with existing code)
SUPABASE_USER = AUTH_USER
SUPABASE_USER_PASSWORD = AUTH_USER_PASSWORD
SUPABASE_AUTH_ENABLED = AUTH_MODE == "jwt"
BASIC_AUTH_ENABLED = False  # No longer used

# Build base URL
# If host starts with http:// or https://, use it directly (may already include port)
# Otherwise, construct the URL with scheme and port
if LIGHTRAG_API_HOST.startswith("http://") or LIGHTRAG_API_HOST.startswith("https://"):
    _base_host = LIGHTRAG_API_HOST.rstrip("/")
    # Check if host already contains a port (e.g., http://host:80)
    _parsed = urlparse(_base_host)
    _host_has_port = _parsed.port is not None
    
    if _host_has_port:
        # Host already has port, use as-is
        LIGHTRAG_API_BASE_URL = f"{_base_host}/rag/v1"
        SUPABASE_AUTH_URL = f"{_base_host}/auth/v1"
    elif LIGHTRAG_API_PORT in (80, 443):
        # Standard ports, don't append
        LIGHTRAG_API_BASE_URL = f"{_base_host}/rag/v1"
        SUPABASE_AUTH_URL = f"{_base_host}/auth/v1"
    else:
        # Append non-standard port
        LIGHTRAG_API_BASE_URL = f"{_base_host}:{LIGHTRAG_API_PORT}/rag/v1"
        SUPABASE_AUTH_URL = f"{_base_host}:{LIGHTRAG_API_PORT}/auth/v1"
else:
    LIGHTRAG_API_BASE_URL = f"http://{LIGHTRAG_API_HOST}:{LIGHTRAG_API_PORT}/rag/v1"
    SUPABASE_AUTH_URL = f"http://{LIGHTRAG_API_HOST}:{LIGHTRAG_API_PORT}/auth/v1"



"""
Auth module for RAG Agent MCP server.

This module handles:
- User login with email/password via /login endpoint
- JWT token management (storage, retrieval)
- Automatic token refresh before expiration

Login endpoint format:
POST /login
Content-Type: application/x-www-form-urlencoded
apikey: {anon_key}

username={email}&password={password}
"""

import asyncio
import logging
import time
from dataclasses import dataclass
from typing import Optional, Tuple

import httpx

logger = logging.getLogger(__name__)


@dataclass
class AuthToken:
    """Represents a Supabase Auth token pair."""
    access_token: str
    refresh_token: str
    expires_at: float  # Unix timestamp
    user_id: Optional[str] = None
    email: Optional[str] = None


class SupabaseAuthClient:
    """Client for Supabase Auth operations.
    
    Handles user authentication and token lifecycle management.
    """
    
    # Refresh token 5 minutes before expiration
    TOKEN_REFRESH_BUFFER_SECONDS = 300
    
    def __init__(
        self,
        auth_url: str,
        anon_key: str,
        user: str,
        password: str,
    ):
        """Initialize Supabase Auth client.
        
        Args:
            auth_url: Supabase Auth API URL (e.g., https://xxx.supabase.co/auth/v1)
            anon_key: Supabase anon key
            user: User email or username
            password: User password
        """
        self.auth_url = auth_url.rstrip("/")
        self.anon_key = anon_key
        self.user = user
        self.password = password
        self._token: Optional[AuthToken] = None
        self._refresh_task: Optional[asyncio.Task] = None
        self._lock = asyncio.Lock()
        
        logger.info(f"Initialized Supabase Auth client for user: {user}")
    
    @property
    def is_authenticated(self) -> bool:
        """Check if client has a valid token."""
        return self._token is not None and not self._is_token_expired()
    
    @property
    def access_token(self) -> Optional[str]:
        """Get current access token."""
        return self._token.access_token if self._token else None
    
    def _is_token_expired(self) -> bool:
        """Check if current token is expired or about to expire."""
        if not self._token:
            return True
        return time.time() >= (self._token.expires_at - self.TOKEN_REFRESH_BUFFER_SECONDS)
    
    async def login(self) -> AuthToken:
        """Login with email/password and obtain tokens.
        
        Calls the /login endpoint with form-urlencoded data.
        
        Returns:
            AuthToken: Token information
            
        Raises:
            httpx.HTTPStatusError: If login fails
        """
        # Use base URL without /auth/v1 suffix, just /login
        base_url = self.auth_url.replace("/auth/v1", "")
        url = f"{base_url}/login"
        
        headers = {
            "apikey": self.anon_key,
            "Content-Type": "application/x-www-form-urlencoded",
        }
        
        # Form data format: username=email&password=password
        form_data = {
            "username": self.user,
            "password": self.password,
        }
        
        # Log request details to stderr for MCP inspector visibility
        import sys
        print("=== LOGIN REQUEST DEBUG ===", file=sys.stderr)
        print(f"URL: {url}", file=sys.stderr)
        print(f"Headers: apikey={self.anon_key[:15]}...{self.anon_key[-10:]}, Content-Type={headers['Content-Type']}", file=sys.stderr)
        print(f"Form data: username={self.user}, password=***", file=sys.stderr)
        print("=== END LOGIN REQUEST DEBUG ===", file=sys.stderr)
        
        async with httpx.AsyncClient() as client:
            response = await client.post(
                url,
                data=form_data,  # Use data= for form-urlencoded
                headers=headers,
                timeout=30.0,
            )
            
            print(f"Login response status: {response.status_code}", file=sys.stderr)
            
            if response.status_code != 200:
                error_detail = response.text
                logger.error(f"Login failed with status {response.status_code}: {error_detail}")
                response.raise_for_status()
            
            data = response.json()
            
            # Calculate expiration time (default 1 hour if not provided)
            # JWT typically has exp claim, but we use a safe default
            expires_in = 3600
            expires_at = time.time() + expires_in
            
            self._token = AuthToken(
                access_token=data["access_token"],
                refresh_token="",  # /login doesn't return refresh_token
                expires_at=expires_at,
                user_id=data.get("user_id"),
                email=data.get("email"),
            )
            
            logger.info(f"Login successful. User ID: {self._token.user_id}, expires in {expires_in}s")
            
            return self._token
    
    async def refresh(self) -> AuthToken:
        """Refresh the access token by re-login.
        
        Since /login doesn't return refresh_token, we perform a full login.
        
        Returns:
            AuthToken: New token information
            
        Raises:
            httpx.HTTPStatusError: If refresh fails
        """
        logger.info("Token expired, performing re-login")
        return await self.login()
    
    async def ensure_valid_token(self) -> str:
        """Ensure we have a valid access token, refreshing if necessary.
        
        Returns:
            str: Valid access token
        """
        async with self._lock:
            if not self._token:
                await self.login()
            elif self._is_token_expired():
                await self.refresh()
            
            return self._token.access_token
    
    async def start_auto_refresh(self) -> None:
        """Start background task to auto-refresh token before expiration."""
        if self._refresh_task and not self._refresh_task.done():
            logger.debug("Auto-refresh task already running")
            return
        
        async def refresh_loop():
            while True:
                try:
                    if self._token:
                        # Calculate time until we should refresh
                        time_until_refresh = (
                            self._token.expires_at 
                            - time.time() 
                            - self.TOKEN_REFRESH_BUFFER_SECONDS
                        )
                        
                        if time_until_refresh > 0:
                            logger.debug(f"Next token refresh in {time_until_refresh:.0f}s")
                            await asyncio.sleep(time_until_refresh)
                        
                        # Refresh token
                        await self.ensure_valid_token()
                    else:
                        # No token yet, wait a bit and check again
                        await asyncio.sleep(60)
                        
                except asyncio.CancelledError:
                    logger.info("Auto-refresh task cancelled")
                    break
                except Exception as e:
                    logger.error(f"Error in auto-refresh loop: {e}")
                    # Wait before retrying
                    await asyncio.sleep(60)
        
        self._refresh_task = asyncio.create_task(refresh_loop())
        logger.info("Started auto-refresh background task")
    
    async def stop_auto_refresh(self) -> None:
        """Stop the auto-refresh background task."""
        if self._refresh_task:
            self._refresh_task.cancel()
            try:
                await self._refresh_task
            except asyncio.CancelledError:
                pass
            self._refresh_task = None
            logger.info("Stopped auto-refresh background task")
    
    async def close(self) -> None:
        """Clean up resources."""
        await self.stop_auto_refresh()
        self._token = None
        logger.info("Supabase Auth client closed")



"""Contains endpoint functions for accessing the API"""

# Импортируем модули для удобства использования
from .get_auth_status_auth_status_get import asyncio_detailed as async_get_auth_status
from .get_auth_status_auth_status_get import sync_detailed as get_auth_status
from .get_status_health_get import asyncio as async_get_health
from .get_status_health_get import sync as get_health
from .login_login_post import asyncio as async_login
from .login_login_post import sync as login
from .redirect_to_webui_get import asyncio_detailed as async_redirect_to_webui
from .redirect_to_webui_get import sync_detailed as redirect_to_webui

__all__ = [
    "get_health",
    "async_get_health",
    "get_auth_status",
    "async_get_auth_status",
    "login",
    "async_login",
    "redirect_to_webui",
    "async_redirect_to_webui",
]



from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.ollama_chat_request_options_type_0 import OllamaChatRequestOptionsType0
    from ..models.ollama_message import OllamaMessage


T = TypeVar("T", bound="OllamaChatRequest")


@_attrs_define
class OllamaChatRequest:
    """
    Attributes:
        model (str):
        messages (list['OllamaMessage']):
        stream (Union[Unset, bool]):  Default: True.
        options (Union['OllamaChatRequestOptionsType0', None, Unset]):
        system (Union[None, Unset, str]):
    """

    model: str
    messages: list["OllamaMessage"]
    stream: Union[Unset, bool] = True
    options: Union["OllamaChatRequestOptionsType0", None, Unset] = UNSET
    system: Union[None, Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.ollama_chat_request_options_type_0 import OllamaChatRequestOptionsType0

        model = self.model

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        stream = self.stream

        options: Union[None, Unset, dict[str, Any]]
        if isinstance(self.options, Unset):
            options = UNSET
        elif isinstance(self.options, OllamaChatRequestOptionsType0):
            options = self.options.to_dict()
        else:
            options = self.options

        system: Union[None, Unset, str]
        if isinstance(self.system, Unset):
            system = UNSET
        else:
            system = self.system

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "model": model,
                "messages": messages,
            }
        )
        if stream is not UNSET:
            field_dict["stream"] = stream
        if options is not UNSET:
            field_dict["options"] = options
        if system is not UNSET:
            field_dict["system"] = system

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.ollama_chat_request_options_type_0 import OllamaChatRequestOptionsType0
        from ..models.ollama_message import OllamaMessage

        d = dict(src_dict)
        model = d.pop("model")

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = OllamaMessage.from_dict(messages_item_data)

            messages.append(messages_item)

        stream = d.pop("stream", UNSET)

        def _parse_options(data: object) -> Union["OllamaChatRequestOptionsType0", None, Unset]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                options_type_0 = OllamaChatRequestOptionsType0.from_dict(data)

                return options_type_0
            except:  # noqa: E722
                pass
            return cast(Union["OllamaChatRequestOptionsType0", None, Unset], data)

        options = _parse_options(d.pop("options", UNSET))

        def _parse_system(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        system = _parse_system(d.pop("system", UNSET))

        ollama_chat_request = cls(
            model=model,
            messages=messages,
            stream=stream,
            options=options,
            system=system,
        )

        ollama_chat_request.additional_properties = d
        return ollama_chat_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



"""
Basic Auth module for RAG Agent MCP server.

This module handles HTTP Basic Authentication for Kong gateway.
"""

import base64
import logging
from dataclasses import dataclass

logger = logging.getLogger(__name__)


@dataclass
class BasicAuthClient:
    """Client for HTTP Basic Authentication.
    
    Generates Authorization header for Kong gateway authentication.
    """
    
    user: str
    password: str
    
    def __post_init__(self):
        """Initialize Basic Auth client."""
        logger.info(f"Initialized Basic Auth client for user: {self.user}")
    
    @property
    def authorization_header(self) -> str:
        """Get the Authorization header value for Basic Auth.
        
        Returns:
            str: Authorization header value (e.g., "Basic dXNlcjpwYXNz")
        """
        credentials = f"{self.user}:{self.password}"
        encoded = base64.b64encode(credentials.encode("utf-8")).decode("utf-8")
        return f"Basic {encoded}"
    
    def get_headers(self) -> dict:
        """Get headers dict with Authorization.
        
        Returns:
            dict: Headers dictionary with Authorization
        """
        return {"Authorization": self.authorization_header}
    
    async def close(self) -> None:
        """Clean up resources (no-op for Basic Auth)."""
        logger.info("Basic Auth client closed")



"""
Entry point for RAG Agent MCP server.
"""

import logging
import sys

from rag_agent import config
from rag_agent.server import mcp

logging.basicConfig(
    level=logging.INFO,
    format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    handlers=[
        logging.StreamHandler(sys.stdout),
    ],
)
logger = logging.getLogger(__name__)


def main():
    """Main function for server startup."""
    try:
        log_level = getattr(logging, "DEBUG")
        logging.getLogger().setLevel(log_level)

        # Print config to stderr for MCP inspector visibility
        print("=== RAG AGENT CONFIG ===", file=sys.stderr)
        print(f"Base URL: {config.LIGHTRAG_API_BASE_URL}", file=sys.stderr)
        print(f"API Key: {config.LIGHTRAG_API_KEY[:20]}...{config.LIGHTRAG_API_KEY[-10:] if config.LIGHTRAG_API_KEY else 'NOT SET'}", file=sys.stderr)
        print(f"Auth Mode: {config.AUTH_MODE}", file=sys.stderr)
        if config.AUTH_USER:
            print(f"Auth User: {config.AUTH_USER}", file=sys.stderr)
        print("=== END CONFIG ===", file=sys.stderr)
        
        logger.info("Starting RAG Agent MCP server")
        logger.info(
            f"RAG Agent API server is expected to be already running and available at: {config.LIGHTRAG_API_BASE_URL}"
        )
        if config.LIGHTRAG_API_KEY:
            logger.info("API key is configured")
        else:
            logger.warning("No API key provided")
        
        # Log authentication mode
        if config.AUTH_MODE == "jwt":
            logger.info(f"Auth mode: Anon Key + JWT for user: {config.AUTH_USER}")
            logger.info(f"Login URL: {config.SUPABASE_AUTH_URL.replace('/auth/v1', '/login')}")
        else:
            logger.info("Auth mode: Service Key (admin privileges)")

        mcp.run(transport="stdio")

    except KeyboardInterrupt:
        logger.info("Server stopped by user")
    except Exception as e:
        logger.exception(f"Error starting server: {str(e)}")
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Contains endpoint functions for accessing the API"""

# Импортируем модули для удобства использования
from .chat_api_chat_post import asyncio as async_chat
from .chat_api_chat_post import sync as chat
from .generate_api_generate_post import asyncio as async_generate
from .generate_api_generate_post import sync as generate
from .get_tags_api_tags_get import asyncio as async_get_tags
from .get_tags_api_tags_get import sync as get_tags
from .get_version_api_version_get import asyncio as async_get_version
from .get_version_api_version_get import sync as get_version

__all__ = [
    "chat",
    "async_chat",
    "generate",
    "async_generate",
    "get_tags",
    "async_get_tags",
    "get_version",
    "async_get_version",
]



"""Contains endpoint functions for accessing the API"""

# Импортируем модули для удобства использования
from .query_text_query_post import asyncio as async_query_document
from .query_text_query_post import sync as query_document
from .query_text_stream_query_stream_post import asyncio as async_query_document_stream
from .query_text_stream_query_stream_post import sync as query_document_stream

__all__ = [
    "query_document",
    "async_query_document",
    "query_document_stream",
    "async_query_document_stream",
]
